/*
 * File: Hanoi.java
 * ----------------
 * This program solves the Tower of Hanoi puzzle using recursion.
 */

import acm.program.*;
import java.util.*;

public class Hanoi extends ConsoleProgram {

/* Standard entry point */
	public static void main(String[] args) {
		new Hanoi().start(args);
	}

/* Program entry point */
	public void run() {
		print("How many disks: ");
		int n = readInt();
		moveTower(n, 'A', 'B', 'C');
	}

/* Method: moveTower(n, start, finish, temp) */
/**
 * Writes out the moves in the Tower of Hanoi puzzle
 * necessary to move a tower of size n from the start
 * spire to the finish spire using the remaining
 * spire for temporary storage.  This implementation
 * simulates the recursive operation using a stack.
 */
	void moveTower(int n, char start, char finish, char temp) {
		Stack<Frame> stack = new Stack<Frame>();
		stack.push(new Frame(n, start, finish, temp, 0));
		int pc = 1;
		while (pc != 0) {
			Frame cf = (Frame) stack.peek();
			if (cf.n == 1) {
				println(cf.start + " -> " + cf.finish);
				pc = 4;
			} else {
				switch (pc) {
				  case 1:
				    stack.push(new Frame(cf.n - 1, cf.start,
				                         cf.temp, cf.finish,
				                         pc + 1));
				    pc = 1;
				    break;
				  case 2:
				    println(cf.start + " -> " + cf.finish);
				    pc = 3;
				    break;
				  case 3:
				    stack.push(new Frame(cf.n - 1, cf.temp,
				                         cf.finish, cf.start,
				                         pc + 1));
				    pc = 1;
				    break;
				}
			}
			if (pc == 4) {
				pc = ((Frame) stack.pop()).pc;
			}
		}
	}
}

/* Private class: Frame */
/**
 * This class collects the data needed for each recursive
 * call to moveTower.  The fields in this structure are
 * precisely those that would be stored in a stack frame.
 */
class Frame {

/* Constructor to create a Frame from its components */
	Frame(int n, char start, char finish, char temp, int pc) {
		this.n = n;
		this.start = start;
		this.finish = finish;
		this.temp = temp;
		this.pc = pc;
	}

/* Fields accessible to the moveTower code */
	int n;
	char start, finish, temp;
	int pc;
}
